/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpemenubar.h>
#include <qpe/config.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include "Atomic.h"
#include "gamewidget.h"

Atomic::Atomic(QWidget *parent, const char *name, WFlags f) :
    QMainWindow(parent, name, f)
{
    /* set window style and caption */
    setCaption("Atomic");

    /* create the play field as the central widget */
    m_gamewidget = new GameWidget(this, "gamewidget");
    setCentralWidget(m_gamewidget);
    m_gamewidget->show();

    /* create the game/animationSpeed submenu */
    m_animationSpeedMenu = new QPopupMenu(0, "animation_speed_menu");
    connect(m_animationSpeedMenu, SIGNAL(activated(int)), this, SLOT(updateAnimationSpeed(int)));
    connect(m_animationSpeedMenu, SIGNAL(activated(int)), m_gamewidget, SLOT(updateAnimationSpeed(int)));
    m_animationSpeedMenu->setCheckable(true);
    m_animationSpeedMenu->insertItem(tr("Slow"), 1);
    m_animationSpeedMenu->insertItem(tr("Normal"), 5);
    m_animationSpeedMenu->insertItem(tr("Fast"), 10);

    /* create the game submenu */
    QPopupMenu *gameMenu = new QPopupMenu(0, "game_menu");
    gameMenu->insertItem(tr("New"), m_gamewidget, SLOT(start()));
    gameMenu->insertSeparator();
    gameMenu->insertItem(tr("Animation Speed"), m_animationSpeedMenu);
    gameMenu->insertSeparator();
    gameMenu->insertItem(tr("Close"), this, SLOT(close()));

    /* create the view submenu */
    QPopupMenu *viewMenu = new QPopupMenu(0, "view_menu");
    viewMenu->insertItem(tr("Board"), 0);
    viewMenu->insertItem(tr("Molecule"), 1);
    connect(viewMenu, SIGNAL(activated(int)), m_gamewidget, SLOT(setView(int)));

    /* create the menu */
    QPEMenuBar *menu = new QPEMenuBar(this);
    menu->insertItem(tr("Game"), gameMenu);
    menu->insertItem(tr("View"), viewMenu);

    /* read the configuration file */
    readConfig();

    /* check the menu entries of game submenus */
    m_animationSpeedMenu->setItemChecked(m_animationSpeed, true);
    m_animationSpeedMenu->setItemChecked(m_animationSpeed, true);
}

Atomic::~Atomic()
{
    writeConfig();
}

void Atomic::readConfig(void)
{
    Config cfg("Atomic");

    cfg.setGroup("Settings");

    int animation_speed = cfg.readNumEntry("AnimationSpeed", 5);
    if (animation_speed <= 1) {
	m_animationSpeed = 1;
    } else if (animation_speed <= 5) {
	m_animationSpeed = 5;
    } else {
	m_animationSpeed = 10;
    }
    m_gamewidget->updateAnimationSpeed(m_animationSpeed);
}

void Atomic::writeConfig(void)
{
    Config cfg("Atomic");

    cfg.setGroup("Settings");
    cfg.writeEntry("AnimationSpeed", m_animationSpeed);
}

void Atomic::updateAnimationSpeed(int animation_speed)
{
    m_animationSpeedMenu->setItemChecked(m_animationSpeed, false);
    m_animationSpeed = animation_speed;
    m_animationSpeedMenu->setItemChecked(m_animationSpeed, true);
    writeConfig();
}

