/* toplevel.cpp

  Copyright (C) 1998   Andreas Wst (AndreasWuest@gmx.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  */

#include "gamewidget.h"
#include "feld.h"
#include "molek.h"

#include <qscrollbar.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qlabel.h>
#include "common.h"

// ##########################
// #	class GameWidget    #
// ##########################

int level;

void GameWidget::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);

    painter.setPen(QColor(0, 0, 0));
    painter.drawText(7, 0, 40, 22, AlignLeft | AlignVCenter, tr("Level: %1").arg(level + 1));
    painter.drawText(54, 0, 50, 22, AlignLeft | AlignVCenter, tr("Moves: %2").arg(m_moves));
}

void GameWidget::keyPressEvent(QKeyEvent *event)
{
    switch (event->key()) {
    case Key_Up:
	feld->startAnimation (Feld::MoveUp);
	break;
    case Key_Down:
	feld->startAnimation (Feld::MoveDown);
	break;
    case Key_Left:
	feld->startAnimation (Feld::MoveLeft);
	break;
    case Key_Right:
	feld->startAnimation (Feld::MoveRight);
	break;
    case Key_N:
    case Key_Space:
    case Key_Enter:
    case Key_PageDown:
	feld->nextAtom();
	break;
    case Key_P:
    case Key_PageUp:
	feld->previousAtom();
	break;
    case Key_B:
	setView(0);
	break;
    case Key_M:
	setView(1);
	break;
    case Key_V:
	setView(m_view ? 0 : 1);
	break;
    default:
	QWidget::keyPressEvent(event);
    }
}

void GameWidget::getButton (int button)
{
    feld->startAnimation ((Feld::Direction)button);
}

void GameWidget::gameOver(int moves) {
    QMessageBox::information(this, tr("Atomic"),
	tr("Game over\nYou solved level %1\nwith %2 moves!").arg(level + 1).arg(moves));
    updateLevel(level+1);
}

void GameWidget::updateMoves(int moves)
{
    m_moves = moves;
    repaint(0, 0, width(), 22);
}

void GameWidget::updateAnimationSpeed(int animationSpeed)
{
    m_animationSpeed = animationSpeed;
    feld->updateAnimationSpeed(animationSpeed);
}

void GameWidget::updateLevel (int l)
{
    if (l >= 0 && l < nlevels) {
	level = l;
    } else {
	level = 0;
    }
    feld->load(level);
    feld->repaint();
    repaint(0, 0, width(), 22);
}

void GameWidget::start(void)
{
    updateLevel(0);
    feld->setMoves(0);
}

void GameWidget::setView(int what)
{
    if (what) {
	molek->setFocus();
	molek->show();
	feld->hide();
	molek->raise();
	m_view = 1;
    } else {
	feld->setFocus();
	feld->show();
	molek->hide();
	feld->raise();
	m_view = 0;
    }
}

GameWidget::GameWidget ( QWidget *parent, const char* name )
    : QWidget( parent, name )
{
    level = 0;
    nlevels = levels->levels();

    scroll = new QScrollBar(0, nlevels - 1, 1, 5, 0, QScrollBar::Horizontal, this, "scrl");
    scroll->move(111, -1);
    scroll->setFixedSize(122, 22);
    connect(scroll, SIGNAL(valueChanged(int)), SLOT (updateLevel(int)));

    // spielfeld
    feld = new Feld (this, "feld");
    feld->setFixedSize(240, 240);
    feld->move(0, 22);
    feld->setFocus();
    feld->show();

    // molecule
    molek = new Molek (this, "molek");
    molek->setFixedSize(240, 240);
    molek->move(0, 22);
    molek->hide();
    feld->setMolek(molek);

    // raise the field widget
    feld->raise();

    connect (feld, SIGNAL (gameOver(int)), SLOT(gameOver(int)));
    connect (feld, SIGNAL (sendMoves(int)), SLOT(updateMoves(int)));

    m_animationSpeed = 5;
    m_moves = 0;
    updateLevel(0);
    m_view = 0;
}

GameWidget::~GameWidget()
{
}

