/****************************************************************
**
** Implementation Molek class, derieved from Qt tutorial 8
**
****************************************************************/

// bemerkungen : wenn paintEvent aufgerufen wird, wird das komplette
//               widget gelscht und nur die sachen gezeichnet, die in
//               paintEvent stehen ! sollen dinge z.b nur bei maustasten-
//               druck gezeichnet werden, so mu dies in mousePressEvent
//               stehen !
//               paintEvent wird aufgerufen, falls fenster berdeckt wird,
//               oder auch einfach bewegt wird


#include "molek.h"
#include <qwidget.h>
#include <ctype.h>
#include "common.h"
#include <qapplication.h>

extern int level;

Molek::Molek( QWidget *parent, const char *name ) : QWidget( parent, name )
{
    setBackgroundColor (QColor (0, 0, 0));
    setMinimumSize(200, 200);
}

Molek::~Molek ()
{
}

const atom& Molek::getAtom(uint index) const
{
  static atom none = { 0, "" };

  if (index > atoms.count() || index == 0)
    return none;

  return *atoms.at(index - 1);
}

void Molek::load (int level)
{
    atoms.clear();

    atom current;
    int atom_index = 1;
    int i;
    int j;
    for (i = 0; levels->atom(level, i) != NULL; i++) {
	QString line(levels->atom(level, i));
	int pos = line.find("-");
	if (pos < 0) {
	    // printf("OOPS\n");
	    continue;
	}
	int num = line.left(pos).toInt();
	if (num == 0) {
	    char c = line[0].latin1();
	    if (c >= 'a' && c <= 'z') {
		num = c - 'a' + 10;
	    } else if (c >= 'A' && c <= 'Z') {
		num = c - 'A' + 10;
	    }
	}
	current.obj = int2atom(num);
	strcpy(current.conn, line.mid(2).ascii());
	if (atoms.find(current) != atoms.end()) {
	    // printf("OOPS\n");
	    continue;
	}
	atoms.append(current);
	atom_index++;
    }
    for (i = 0; i < 10; i++) {
	QString line(levels->mole(level, i));
	for (j = 0; j < 10; j++) {
	    molek[j][i] = atom2int(line.at(j).latin1());
	}
    }

    mname = qApp->translate("@default", levels->name(level));

    int& height = _size.rheight();
    int& width = _size.rwidth();

    height = 0;
    width = 0;

    for (int i = 0; i < 10; i++)
		for (int j = 0; j < 10; j++) {
			if (molek [i][j] == 0)
				continue;
			if (i > width) width = i;
			if (j > height) height = j;
		}
    height++;
    width++;

    repaint ();
}

void Molek::paintEvent( QPaintEvent * )
{
    QString st = QString("Level: %1").arg(level+1);

    QPainter paint (this);
    paint.setPen (QColor (190, 190, 190));
    paint.drawText (7, height() - 36, mname);
    paint.drawText (7, height() - 18, st);
    // spielfeld gleich zeichnen
    for (int i = 0; i < 10; i++)
		for (int j = 0; j < 10; j++) {
			int x = 7 + i * 16;
			int y = 7 + j * 16;

			if (molek[i][j] == 0)
				continue;

			// paints atoms
			if (getAtom(molek [i] [j]).obj <= '9' && getAtom(molek [i] [j]).obj >= '1')
				paint.drawPixmap(x, y, sprites->m_atom[getAtom(molek[i][j]).obj - '1']);
			else if (getAtom(molek [i] [j]).obj == 'a')
				paint.drawPixmap(x, y, sprites->m_atom[9]);

			// paints cristals
			if (getAtom(molek [i] [j]).obj == 'o')
				paint.drawPixmap(x, y, sprites->m_crystal);

			// paints connections
			if (isdigit(getAtom(molek[i][j]).obj) || getAtom(molek[i][j]).obj == 'o')
				for (int c = 0; c < MAX_CONNS_PER_ATOM; c++) {
					char conn = getAtom(molek [i] [j]).conn[c];
					if (!conn)
						break;

					if (conn >= 'a' && conn <= 'a' + 8)
						paint.drawPixmap(x, y, sprites->m_conn[conn - 'a']);
					else
						paint.drawPixmap(x, y, sprites->m_conn[conn - 'A' + 8]);

				}


			// paints connections
			if (getAtom(molek[i][j]).obj >= 'A' && getAtom(molek[i][j]).obj <= 'F')
				switch (getAtom(molek[i][j]).obj - 'A') {
				case 0: paint.drawPixmap(x, y, sprites->m_horizontal); break;
				case 1: paint.drawPixmap(x, y, sprites->m_bottomleft2topright); break;
				case 2: paint.drawPixmap(x, y, sprites->m_vertical); break;
				case 3: paint.drawPixmap(x, y, sprites->m_topleft2bottomright); break;
				}

		}

    paint.end ();
}
